// ██╗███╗   ███╗██████╗  ██████╗ ██████╗ ████████╗███████╗
// ██║████╗ ████║██╔══██╗██╔═══██╗██╔══██╗╚══██╔══╝██╔════╝
// ██║██╔████╔██║██████╔╝██║   ██║██████╔╝   ██║   ███████╗
// ██║██║╚██╔╝██║██╔═══╝ ██║   ██║██╔══██╗   ██║   ╚════██║
// ██║██║ ╚═╝ ██║██║     ╚██████╔╝██║  ██║   ██║   ███████║
// ╚═╝╚═╝     ╚═╝╚═╝      ╚═════╝ ╚═╝  ╚═╝   ╚═╝   ╚══════╝
import { ActionFormData, ModalFormData, FormCancelationReason } from "@minecraft/server-ui";
import { Database } from "index.js";
import { isitem, isblock } from "texturePath.js";
import { world, system } from '@minecraft/server';

//  ██████╗██████╗  █████╗ ████████╗███████╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝
// ██║     ██████╔╝███████║   ██║   █████╗  
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝  
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝
//  ██████╗ ██████╗ ███╗   ██╗███████╗████████╗███████╗
// ██╔════╝██╔═══██╗████╗  ██║██╔════╝╚══██╔══╝██╔════╝
// ██║     ██║   ██║██╔██╗ ██║███████╗   ██║   ███████╗
// ██║     ██║   ██║██║╚██╗██║╚════██║   ██║   ╚════██║
// ╚██████╗╚██████╔╝██║ ╚████║███████║   ██║   ███████║
//  ╚═════╝ ╚═════╝ ╚═╝  ╚═══╝╚══════╝   ╚═╝   ╚══════╝
const SettingsDb = new Database("Settings");
const ItemsDb = new Database("Items");
const cooldowns = new Map();
const prizes = [
    { item: 'iron_ingot', quantity: 10, weight: 40 },// Common
    { item: 'emerald', quantity: 5, weight: 30 },    // Uncommon
    { item: 'gold_ingot', quantity: 7, weight: 35 }, // Uncommon
    { item: 'nether_star', quantity: 1, weight: 5 }, // Rare
    { item: 'diamond_sword', quantity: 1, weight: 10 }, // Rare
    { item: 'cake', quantity: 1, weight: 3 }, // Ultra Rare
    { item: 'blaze_rod', quantity: 1, weight: 1 }, // Legendary
    { item: 'netherite_leggings', quantity: 1, weight: 1 }, // Legendary
    { item: 'netherite_pickaxe', quantity: 1, weight: 1 }, // Legendary
];

function setCooldown(player) {
    cooldowns.set(player, Date.now());
}

//  ██████╗██╗  ██╗ █████╗ ████████╗     ██████╗ ██████╗ ███╗   ███╗███╗   ███╗ █████╗ ███╗   ██╗██████╗ 
// ██╔════╝██║  ██║██╔══██╗╚══██╔══╝    ██╔════╝██╔═══██╗████╗ ████║████╗ ████║██╔══██╗████╗  ██║██╔══██╗
// ██║     ███████║███████║   ██║       ██║     ██║   ██║██╔████╔██║██╔████╔██║███████║██╔██╗ ██║██║  ██║
// ██║     ██╔══██║██╔══██║   ██║       ██║     ██║   ██║██║╚██╔╝██║██║╚██╔╝██║██╔══██║██║╚██╗██║██║  ██║
// ╚██████╗██║  ██║██║  ██║   ██║       ╚██████╗╚██████╔╝██║ ╚═╝ ██║██║ ╚═╝ ██║██║  ██║██║ ╚████║██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝        ╚═════╝ ╚═════╝ ╚═╝     ╚═╝╚═╝     ╚═╝╚═╝  ╚═╝╚═╝  ╚═══╝╚═════╝ 
system.runInterval(() => {
    const ToggleCrateChatcommand = SettingsDb.get('KittysCrateSettings:ToggleCrateChatcommand') || false;
    if (ToggleCrateChatcommand === 'true' || ToggleCrateChatcommand === true) {
        return;
    }
    const Crate_Command = SettingsDb.get('KittysCrateSettings:Crate_Command') || '!crate';
    world.getDimension('overworld').runCommand(`say Open the crate menu with §6${Crate_Command}`);
}, 15000);

world.beforeEvents.chatSend.subscribe(async (data) => {
    const ToggleCrateChatcommand = SettingsDb.get('KittysCrateSettings:ToggleCrateChatcommand') || false;
    if (ToggleCrateChatcommand === 'true' || ToggleCrateChatcommand === true) {
        return;
    }
    const Crate_Command = SettingsDb.get('KittysCrateSettings:Crate_Command') || '!crate';
    const crateTimerValue = SettingsDb.get('KittysCrateSettings:CrateTimer') || '1';
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    if (!data.message.startsWith(Crate_Command)) return;
    data.cancel = true;

    if (
        data.sender.hasTag("Admin") ||
        data.sender.hasTag("admin") ||
        data.sender.hasTag("Staff") ||
        data.sender.hasTag("staff") ||
        data.sender.hasTag("Owner") ||
        data.sender.hasTag("owner")
    ) {
        await Admin_Main_Menu(data.sender);
        return;
    }

    function isOnCooldown(player) {
        const now = Date.now();
        if (!cooldowns.has(player.name)) return false;
        const cooldown = cooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }

    if (isOnCooldown(data.sender)) {
        const remainingTime = cooldownTime - (Date.now() - cooldowns.get(data.sender.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.sender.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the crate again.✨`);
        data.sender.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await data.sender.playSound('note.pling', { volume: 1 });


    const crateMenu = new ActionFormData()
        .title('!Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.sender, crateMenu);
        result.selection === 0 ? await Crate_Form(data.sender) : await closeCrateMenu(data.sender);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});

//  ██████╗██████╗  █████╗ ████████╗███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ██║     ██████╔╝███████║   ██║   █████╗      ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                   
async function Crate_Form(player) {
    const crateTimerValue = SettingsDb.get('KittysCrateSettings:CrateTimer') || '1';
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds

    function isOnCooldown(player) {
        const now = Date.now();
        if (!cooldowns.has(player.name)) return false;
        const cooldown = cooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }
     if (
        player.hasTag("Admin") ||
        player.hasTag("admin") ||
        player.hasTag("Staff") ||
        player.hasTag("staff") ||
        player.hasTag("Owner") ||
        player.hasTag("owner")
    ) {
        const allPrizes = getPrizes();
        const randomPrize = getRandomPrize(allPrizes);
        await givePlayerPrize(player, randomPrize);
        await Crate_Result_Form(player, randomPrize, allPrizes);
        return;
    }
    if (isOnCooldown(player)) {
        const remainingTime = cooldownTime - (Date.now() - cooldowns.get(player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the crate again.✨`);
        player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    const allPrizes = getPrizes();
    const randomPrize = getRandomPrize(allPrizes);
    await givePlayerPrize(player, randomPrize);
    await Crate_Result_Form(player, randomPrize, allPrizes);
    cooldowns.set(player.name, Date.now());
}

const getPrizes = () => {
    const customPrizes = ItemsDb.getAll();
    if (customPrizes.length === 0) {
        return prizes;
    }
    return customPrizes.map(itemKey => {
        const itemData = JSON.parse(ItemsDb.get(itemKey));
        return {
            item: itemKey,
            quantity: itemData.itemValue || 1,
            weight: itemData.weight || 1,
        };
    });
};

function getRandomPrize(prizes) {
    const totalWeight = prizes.reduce((sum, prize) => sum + prize.weight, 0);
    const randomNum = Math.floor(Math.random() * totalWeight);

    let runningSum = 0;
    for (const prize of prizes) {
        runningSum += prize.weight;
        if (randomNum < runningSum) return prize;
    }
}

async function givePlayerPrize(player, prize) {
    const overworld = world.getDimension('overworld');
    await overworld.runCommandAsync(`give "${player.name}" ${prize.item} ${prize.quantity}`);
    setCooldown(player.name);
    player.playSound('kittys.crate.open', { volume: 1 });
}

//  ██████╗██████╗  █████╗ ████████╗███████╗    ██████╗ ███████╗███████╗██╗   ██╗██╗  ████████╗    ███████╗ ██████╗ ██████╗ ███╗   ███╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔══██╗██╔════╝██╔════╝██║   ██║██║  ╚══██╔══╝    ██╔════╝██╔═══██╗██╔══██╗████╗ ████║
// ██║     ██████╔╝███████║   ██║   █████╗      ██████╔╝█████╗  ███████╗██║   ██║██║     ██║       █████╗  ██║   ██║██████╔╝██╔████╔██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██╔══██╗██╔══╝  ╚════██║██║   ██║██║     ██║       ██╔══╝  ██║   ██║██╔══██╗██║╚██╔╝██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║  ██║███████╗███████║╚██████╔╝███████╗██║       ██║     ╚██████╔╝██║  ██║██║ ╚═╝ ██║
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝  ╚═╝╚══════╝╚══════╝ ╚═════╝ ╚══════╝╚═╝       ╚═╝      ╚═════╝ ╚═╝  ╚═╝╚═╝     ╚═╝                                                                                                                                                                                                                                                             
async function Crate_Result_Form(player, wonPrize, allPrizes) {
    const spinResultForm = new ActionFormData()
        .title(' !Crate Opened!')

    let texturePath;
    if (isitem.has(wonPrize.item)) {
        texturePath = `textures/items/${wonPrize.item}`;
    } else if (isblock.has(wonPrize.item)) {
        texturePath = `textures/blocks/${wonPrize.item}`;
    } else {
        texturePath = `textures/ui/gift_square`;
    }
    
    spinResultForm.button(`You Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''}!✨`, texturePath);
    const ToggleCrateAnnouncementcommand = SettingsDb.get('KittysCrateSettings:ToggleCrateAnnouncementcommand') || false;
    if (ToggleCrateAnnouncementcommand === 'false' || ToggleCrateAnnouncementcommand === false) {
        world.sendMessage(`${player.name} Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''} From The Crate!✨`);
    }
    allPrizes
        .filter(prize => prize !== wonPrize)
        .forEach(prize => {
            let texturePath;
            if (isitem.has(prize.item)) {
                texturePath = `textures/items/${prize.item}`;
            } else if (isblock.has(prize.item)) {
                texturePath = `textures/blocks/${prize.item}`;
            } else {
                texturePath = `textures/ui/gift_square`;
            }

            spinResultForm.button(`${prize.quantity} ${prize.item.replace(/_/g, ' ')}${prize.quantity > 1 ? 's' : ''}✨`, texturePath);
        });

    spinResultForm.button('Close');

    await spinResultForm.show(player);
    try {
        await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
        await player.playSound('note.bass', { volume: 0.5 });
    } catch (error) {
        console.error(`Error displaying spin result: ${error.message}`);
    }
}

async function closeCrateMenu(player) {
    await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
    await player.playSound('note.bass', { volume: 0.5 });
}

async function forceShow(player, form, timeout = Infinity) {
    const startTick = system.currentTick;

    while ((system.currentTick - startTick) < timeout) {
        const result = await form.show(player);
        if (result.cancelationReason !== FormCancelationReason.UserBusy) return result;
    }

    throw new Error(`Timed out after ${timeout} ticks`);
}

const getSettingsDb = () => {
    const savedSettings = {
        ToggleCrateChatcommand: SettingsDb.get('KittysCrateSettings:ToggleCrateChatcommand') || false,
        ToggleCrateAnnouncementcommand: SettingsDb.get('KittysCrateSettings:ToggleCrateAnnouncementcommand') || false,
        CrateTimer: SettingsDb.get('KittysCrateSettings:CrateTimer') || '1',
        Crate_Command: SettingsDb.get('KittysCrateSettings:Crate_Command') || '!crate',
    };
    return savedSettings;
};

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝ 
async function Admin_Main_Menu(sender) {
    await sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await sender.playSound('note.pling', { volume: 1 });

    const crateMenu = new ActionFormData()
        .title('Admin !Crate')
        .body('Admin !Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel')
        .button('Settings', 'textures/ui/gear');

    try {
        const result = await forceShow(sender, crateMenu);
        if (result.selection === 0) {
            await Crate_Form(sender);
        } else if (result.selection === 2) {
            await Admin_Crate_Settings_Main(sender);
        } else {
            await closeCrateMenu(sender);
        }
    } catch (error) {
        console.error(`Error displaying crate menu: ${error.message}`);
    }
}

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗     ██████╗██████╗  █████╗ ████████╗███████╗    ███████╗███████╗████████╗████████╗██╗███╗   ██╗ ██████╗ ███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔════╝██╔════╝╚══██╔══╝╚══██╔══╝██║████╗  ██║██╔════╝ ██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██║     ██████╔╝███████║   ██║   █████╗      ███████╗█████╗     ██║      ██║   ██║██╔██╗ ██║██║  ███╗███████╗    ██╔████╔██║███████║██║██╔██╗ ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ╚════██║██╔══╝     ██║      ██║   ██║██║╚██╗██║██║   ██║╚════██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ███████║███████╗   ██║      ██║   ██║██║ ╚████║╚██████╔╝███████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝     ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚══════╝╚══════╝   ╚═╝      ╚═╝   ╚═╝╚═╝  ╚═══╝ ╚═════╝ ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝                                                                                                                                                                                      
async function Admin_Crate_Settings_Main(player) {
    const customUi = new ActionFormData()
        .title("Kittys Crate Settings Main Menu")
        .body("")
        .button("Crate Settings", "textures/ui/accessibility_glyph_color")
        .button("Add Item", "textures/ui/icon_best3")
        .button("Remove Item", "textures/ui/enable_editor");

    customUi.show(player).then(response => {

        switch (response.selection) {

            case 0:
                const savedSettings = getSettingsDb();
                const CrateTimerValue = Number(savedSettings.CrateTimer);
                const ToggleCrateChatcommandValue = savedSettings.ToggleCrateChatcommand === 'true' ? true : false;
                const ToggleCrateAnnouncementcommandValue = savedSettings.ToggleCrateAnnouncementcommand === 'true' ? true : false;

                const form = new ModalFormData()
                    .title('Kittys Crate Settings Menu')
                    .slider(`Crate Reset Time (Hours)✨`, 1, 24, 1, CrateTimerValue)
                    .textField('Set The Crate Command,✨\nThis Will Be The New Command To Open The Crate Menu', '!crate', savedSettings.Crate_Command)
                    .toggle(`Disable ${savedSettings.Crate_Command} Chat Command`, ToggleCrateChatcommandValue)
                    .toggle(`Disable !crate Chat Prize Announcement`, ToggleCrateAnnouncementcommandValue);

                form.show(player).then(response => {
                    const [CrateTimer, Crate_Command, ToggleCrateChatcommand, ToggleCrateAnnouncementcommand] = response.formValues;
                    // Validate Crate_Command to ensure it's a non-empty string
                    if (!Crate_Command || typeof Crate_Command !== 'string') {
                        world.sendMessage("⚠️ Please provide a valid crate command.");
                        player.playSound('note.bass', { volume: 0.5 });
                        return Admin_Main_Menu(player);
                    }
                    SettingsDb.set('KittysCrateSettings:CrateTimer', `${CrateTimer}`);
                    SettingsDb.set('KittysCrateSettings:Crate_Command', Crate_Command);
                    SettingsDb.set('KittysCrateSettings:ToggleCrateChatcommand', `${ToggleCrateChatcommand}`);
                    SettingsDb.set('KittysCrateSettings:ToggleCrateAnnouncementcommand', `${ToggleCrateAnnouncementcommand}`);

                    return Admin_Main_Menu(player);
                });
                break;

            //  █████╗ ██████╗ ██████╗     ██╗████████╗███████╗███╗   ███╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔══██╗██╔══██╗    ██║╚══██╔══╝██╔════╝████╗ ████║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ███████║██║  ██║██║  ██║    ██║   ██║   █████╗  ██╔████╔██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██║██║  ██║██║  ██║    ██║   ██║   ██╔══╝  ██║╚██╔╝██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║██████╔╝██████╔╝    ██║   ██║   ███████╗██║ ╚═╝ ██║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚═════╝ ╚═════╝     ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                     
            case 1:
                (async () => {
                    const form = new ModalFormData()
                        .title('Kittys Add Item Menu')
                        .textField('Add Item\nThis Will Display On The Crate Prize Screen:✨', 'apple, iron_sword')
                        .slider(`Quantity✨`, 1, 64, 1, 1)
                        .slider(`0 = Rare\n100 = Common\nRarity✨`, 1, 100, 1, 1);

                    const response = await form.show(player);
                    try {
                        const itemName = response.formValues[0];
                        if (!itemName) {
                            world.sendMessage("Please enter a valid item name.✨");
                            player.playSound('note.bass', { volume: 0.5 });
                            return;
                        }

                        const itemData = {
                            itemValue: response.formValues[1], // Quantity
                            rarity: response.formValues[2] // Rarity
                        };

                        ItemsDb.set(`${itemName}`, JSON.stringify(itemData));
                        world.sendMessage(`Item added: ${itemName} (Quantity: ${itemData.itemValue}, Rarity: ${itemData.rarity})✨`);
                        player.playSound('random.levelup', { volume: 0.8 });
                        return Admin_Main_Menu(player);

                    } catch (error) {
                        world.sendMessage(`Failed to add item: ${error.message}✨`);
                        player.playSound('note.bass', { volume: 0.5 });
                    }
                })();
                break;

            // ██████╗ ███████╗███╗   ███╗ ██████╗ ██╗   ██╗███████╗    ██╗████████╗███████╗███╗   ███╗███████╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔════╝████╗ ████║██╔═══██╗██║   ██║██╔════╝    ██║╚══██╔══╝██╔════╝████╗ ████║██╔════╝    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ██████╔╝█████╗  ██╔████╔██║██║   ██║██║   ██║█████╗      ██║   ██║   █████╗  ██╔████╔██║███████╗    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██╗██╔══╝  ██║╚██╔╝██║██║   ██║╚██╗ ██╔╝██╔══╝      ██║   ██║   ██╔══╝  ██║╚██╔╝██║╚════██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║███████╗██║ ╚═╝ ██║╚██████╔╝ ╚████╔╝ ███████╗    ██║   ██║   ███████╗██║ ╚═╝ ██║███████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚══════╝╚═╝     ╚═╝ ╚═════╝   ╚═══╝  ╚══════╝    ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝╚══════╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                                          
            case 2:
                (async () => {
                    const allItemsDb = ItemsDb.getAll();
                    if (allItemsDb.length === 0) {
                        world.sendMessage("No Items to remove.✨");
                        player.playSound('note.bass', { volume: 0.5 });
                        return;
                    }
                    let page = 0;
                    const totalPages = Math.ceil(allItemsDb.length / 5);
                    while (true) {
                        const start = page * 5;
                        const end = start + 5;

                        const form = new ActionFormData()
                            .title("Kittys Remove Items Menu")
                            .body(`Select an Item to Remove:\nPage: ${page + 1} of ${totalPages}✨`);
                        allItemsDb.slice(start, end).forEach((itemKey) => {
                            const itemData = JSON.parse(ItemsDb.get(itemKey));
                            const itemQuantity = itemData.itemValue;
                            const itemRarity = itemData.rarity; // Extract rarity

                            let texturePath;
                            if (isitem.has(itemKey)) {
                                texturePath = `textures/items/${itemKey}`;
                            } else if (isblock.has(itemKey)) {
                                texturePath = `textures/blocks/${itemKey}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            form.button(` §l§cRemove:\n §l§r${itemKey} (x${itemQuantity}) \n(Rarity: ${itemRarity})✨`, texturePath);
                        });
                        if (page > 0) {
                            form.button(` §o§1Last Page\n §3[ Page: ${page} ]✨`, "textures/ui/generic_dpad_left");
                        }
                        if (end < allItemsDb.length) {
                            form.button(` §o§1Next Page\n §3[ Page: ${page + 2} ]✨`, "textures/ui/generic_dpad_right");
                        }
                        form.button("§l§cMain\nMenu✨", "textures/ui/icon_map");

                        const response = await form.show(player);
                        const selectedItemIndex = response.selection;
                        const totalButtonsOnPage = allItemsDb.slice(start, end).length;
                        const hasPreviousButton = page > 0 ? 1 : 0;
                        const hasNextButton = end < allItemsDb.length ? 1 : 0;
                        if (selectedItemIndex === totalButtonsOnPage + hasPreviousButton + hasNextButton) {
                            return Admin_Main_Menu(player);
                        }
                        if (page > 0 && selectedItemIndex === totalButtonsOnPage) {
                            page--;
                            continue;
                        }
                        if (end < allItemsDb.length && selectedItemIndex === totalButtonsOnPage + hasPreviousButton) {
                            page++;
                            continue;
                        }
                        const itemToRemove = allItemsDb[start + selectedItemIndex];
                        const itemData = JSON.parse(ItemsDb.get(itemToRemove));
                        const itemQuantity = itemData.itemValue;
                        const itemRarity = itemData.rarity;
                        try {
                            let texturePath;
                            if (isitem.has(itemToRemove)) {
                                texturePath = `textures/items/${itemToRemove}`;
                            } else if (isblock.has(itemToRemove)) {
                                texturePath = `textures/blocks/${itemToRemove}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            //  ██████╗ ██████╗ ███╗   ██╗███████╗██╗██████╗ ███╗   ███╗    ██╗████████╗███████╗███╗   ███╗    ██████╗ ███████╗███╗   ███╗ ██████╗ ██╗   ██╗ █████╗ ██╗         ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
                            // ██╔════╝██╔═══██╗████╗  ██║██╔════╝██║██╔══██╗████╗ ████║    ██║╚══██╔══╝██╔════╝████╗ ████║    ██╔══██╗██╔════╝████╗ ████║██╔═══██╗██║   ██║██╔══██╗██║         ████╗ ████║██╔════╝████╗  ██║██║   ██║
                            // ██║     ██║   ██║██╔██╗ ██║█████╗  ██║██████╔╝██╔████╔██║    ██║   ██║   █████╗  ██╔████╔██║    ██████╔╝█████╗  ██╔████╔██║██║   ██║██║   ██║███████║██║         ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
                            // ██║     ██║   ██║██║╚██╗██║██╔══╝  ██║██╔══██╗██║╚██╔╝██║    ██║   ██║   ██╔══╝  ██║╚██╔╝██║    ██╔══██╗██╔══╝  ██║╚██╔╝██║██║   ██║╚██╗ ██╔╝██╔══██║██║         ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
                            // ╚██████╗╚██████╔╝██║ ╚████║██║     ██║██║  ██║██║ ╚═╝ ██║    ██║   ██║   ███████╗██║ ╚═╝ ██║    ██║  ██║███████╗██║ ╚═╝ ██║╚██████╔╝ ╚████╔╝ ██║  ██║███████╗    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
                            //  ╚═════╝ ╚═════╝ ╚═╝  ╚═══╝╚═╝     ╚═╝╚═╝  ╚═╝╚═╝     ╚═╝    ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝    ╚═╝  ╚═╝╚══════╝╚═╝     ╚═╝ ╚═════╝   ╚═══╝  ╚═╝  ╚═╝╚══════╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝ 
                            const ConfirmForm = new ActionFormData()
                                .title('Confirm Item Removal')
                                .body(`Are you sure you want to remove the item: §l${itemToRemove} (x${itemQuantity}) (Rarity: ${itemRarity})§r?✨`)
                                .button("Go Back\n[ Clear Item ]✨", "textures/ui/icon_map")
                                .button(`Remove: §l${itemToRemove} (x${itemQuantity}) (Rarity: ${itemRarity})§r?✨`, texturePath)
                                .button("Go Back\n[ Clear Item ]✨", "textures/ui/icon_map");

                            const confirmResponse = await ConfirmForm.show(player);

                            if (confirmResponse.selection === 1) {
                                try {
                                    ItemsDb.del(itemToRemove);
                                    world.sendMessage(`Item removed: ${itemToRemove} (x${itemQuantity}) (Rarity: ${itemRarity})✨`);
                                    player.playSound('note.bass', { volume: 0.5 });
                                    return Admin_Main_Menu(player);
                                } catch (error) {
                                    console.warn(error);
                                    world.sendMessage("Error occurred while removing item.✨");
                                    player.playSound('note.bass', { volume: 0.5 });
                                }
                            }
                        } catch (error) {
                            world.sendMessage("Error occurred while showing the confirmation dialog.✨");
                            player.playSound('note.bass', { volume: 0.5 });
                        }
                    }
                })();
                break;
        }
    });
}

// ██╗    ██╗ ██████╗ ██████╗ ██╗     ██████╗     ███████╗██╗   ██╗███████╗███╗   ██╗████████╗███████╗
// ██║    ██║██╔═══██╗██╔══██╗██║     ██╔══██╗    ██╔════╝██║   ██║██╔════╝████╗  ██║╚══██╔══╝██╔════╝
// ██║ █╗ ██║██║   ██║██████╔╝██║     ██║  ██║    █████╗  ██║   ██║█████╗  ██╔██╗ ██║   ██║   ███████╗
// ██║███╗██║██║   ██║██╔══██╗██║     ██║  ██║    ██╔══╝  ╚██╗ ██╔╝██╔══╝  ██║╚██╗██║   ██║   ╚════██║
// ╚███╔███╔╝╚██████╔╝██║  ██║███████╗██████╔╝    ███████╗ ╚████╔╝ ███████╗██║ ╚████║   ██║   ███████║
//  ╚══╝╚══╝  ╚═════╝ ╚═╝  ╚═╝╚══════╝╚═════╝     ╚══════╝  ╚═══╝  ╚══════╝╚═╝  ╚═══╝   ╚═╝   ╚══════╝
const Chests = new Set()

world.afterEvents.entitySpawn.subscribe((data) => {
    if (data.entity.typeId != "kittys:chest") return
    const chest = data.entity
    Chests.add(chest)
    system.runTimeout(() => {
        if (!chest.isValid()) return
        chest.nameTag = `Crate`
    }, 10)
})

world.beforeEvents.playerInteractWithEntity.subscribe(async (data) => {
    const { player, target } = data
    if (target.typeId != "kittys:chest") return
    const crateTimerValue = SettingsDb.get('KittysCrateSettings:CrateTimer') || '1';
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    data.cancel = true;
    system.run(() => {
        target.dimension.playSound("random.pop", target.location, { pitch: 0.5 })
    })
    if (
        data.player.hasTag("Admin") ||
        data.player.hasTag("admin") ||
        data.player.hasTag("Staff") ||
        data.player.hasTag("staff") ||
        data.player.hasTag("Owner") ||
        data.player.hasTag("owner")
    ) {
        await Admin_Main_Menu(data.player);
        return;
    }

    function isOnCooldown(player) {
        const now = Date.now();
        if (!cooldowns.has(player.name)) return false;
        const cooldown = cooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }

    if (isOnCooldown(data.player)) {
        const remainingTime = cooldownTime - (Date.now() - cooldowns.get(data.player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the crate again.✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.player.sendMessage('§6§lOpening Crate Menu!✨');


    const crateMenu = new ActionFormData()
        .title('!Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.player, crateMenu);
        result.selection === 0 ? await Crate_Form(data.player) : await closeCrateMenu(data.player);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});

// ██╗     ███████╗ ██████╗ ███████╗███╗   ██╗██████╗  █████╗ ██████╗ ██╗   ██╗     ██████╗██████╗  █████╗ ████████╗███████╗
// ██║     ██╔════╝██╔════╝ ██╔════╝████╗  ██║██╔══██╗██╔══██╗██╔══██╗╚██╗ ██╔╝    ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝
// ██║     █████╗  ██║  ███╗█████╗  ██╔██╗ ██║██║  ██║███████║██████╔╝ ╚████╔╝     ██║     ██████╔╝███████║   ██║   █████╗  
// ██║     ██╔══╝  ██║   ██║██╔══╝  ██║╚██╗██║██║  ██║██╔══██║██╔══██╗  ╚██╔╝      ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝  
// ███████╗███████╗╚██████╔╝███████╗██║ ╚████║██████╔╝██║  ██║██║  ██║   ██║       ╚██████╗██║  ██║██║  ██║   ██║   ███████╗
// ╚══════╝╚══════╝ ╚═════╝ ╚══════╝╚═╝  ╚═══╝╚═════╝ ╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝        ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝                                                                                                                    
//  ██████╗ ██████╗ ███╗   ██╗███████╗████████╗███████╗
// ██╔════╝██╔═══██╗████╗  ██║██╔════╝╚══██╔══╝██╔════╝
// ██║     ██║   ██║██╔██╗ ██║███████╗   ██║   ███████╗
// ██║     ██║   ██║██║╚██╗██║╚════██║   ██║   ╚════██║
// ╚██████╗╚██████╔╝██║ ╚████║███████║   ██║   ███████║
//  ╚═════╝ ╚═════╝ ╚═╝  ╚═══╝╚══════╝   ╚═╝   ╚══════╝
const LegendarySettingsDb = new Database("LegendarySettings");
const LegendaryItemsDb = new Database("LegendaryItems");
const Legendarycooldowns = new Map();
const Legendaryprizes = [
    { item: 'netherite_sword', quantity: 1, weight: 5 },  // Legendary weapon
    { item: 'elytra', quantity: 1, weight: 3 },           // Rare flying item
    { item: 'ender_pearl', quantity: 2, weight: 2 }, // Powerful healing item
    { item: 'end_crystal', quantity: 1, weight: 1 },
    { item: 'beacon', quantity: 1, weight: 2 },           // High-value structure block
    { item: 'netherite_block', quantity: 1, weight: 1 },  // Extremely rare block material
    { item: 'dragon_egg', quantity: 1, weight: 0.5 },     // Unique decorative item
    { item: 'record_pigstep', quantity: 1, weight: 4 },      // Legendary
    { item: 'trident', quantity: 1, weight: 3 },          // Legendary weapon
    { item: 'end_crystal', quantity: 5, weight: 1 },      // High-value item for reviving Ender Dragon
];

function LegendarysetCooldown(player) {
    Legendarycooldowns.set(player, Date.now());
}

//  ██████╗██╗  ██╗ █████╗ ████████╗     ██████╗ ██████╗ ███╗   ███╗███╗   ███╗ █████╗ ███╗   ██╗██████╗ 
// ██╔════╝██║  ██║██╔══██╗╚══██╔══╝    ██╔════╝██╔═══██╗████╗ ████║████╗ ████║██╔══██╗████╗  ██║██╔══██╗
// ██║     ███████║███████║   ██║       ██║     ██║   ██║██╔████╔██║██╔████╔██║███████║██╔██╗ ██║██║  ██║
// ██║     ██╔══██║██╔══██║   ██║       ██║     ██║   ██║██║╚██╔╝██║██║╚██╔╝██║██╔══██║██║╚██╗██║██║  ██║
// ╚██████╗██║  ██║██║  ██║   ██║       ╚██████╗╚██████╔╝██║ ╚═╝ ██║██║ ╚═╝ ██║██║  ██║██║ ╚████║██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝        ╚═════╝ ╚═════╝ ╚═╝     ╚═╝╚═╝     ╚═╝╚═╝  ╚═╝╚═╝  ╚═══╝╚═════╝ 
system.runInterval(() => {
    const ToggleLegendaryCrateChatcommand = LegendarySettingsDb.get('LegendaryKittysCrateSettings:ToggleLegendaryCrateChatcommand') || false;
    if (ToggleLegendaryCrateChatcommand === 'true' || ToggleLegendaryCrateChatcommand === true) {
        return;
    }
    const LegendaryCrate_Command = LegendarySettingsDb.get('LegendaryKittysCrateSettings:LegendaryCrate_Command') || '!legendarycrate';
    world.getDimension('overworld').runCommand(`say Open the crate menu with §6${LegendaryCrate_Command}`);
}, 15000);

world.beforeEvents.chatSend.subscribe(async (data) => {
    const Crate_Command = LegendarySettingsDb.get('LegendaryKittysCrateSettings:LegendaryCrate_Command') || '!legendarycrate';
    const crateTimerValue = LegendarySettingsDb.get('LegendaryKittysCrateSettings:LegendaryCrateTimer') || '2';
    const ToggleLegendaryCrateChatcommand = LegendarySettingsDb.get('LegendaryKittysCrateSettings:ToggleLegendaryCrateChatcommand') || false;
    if (ToggleLegendaryCrateChatcommand === 'true' || ToggleLegendaryCrateChatcommand === true) {
        return;
    }
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    if (!data.message.startsWith(Crate_Command)) return;
    data.cancel = true;

    if (
        data.sender.hasTag("Admin") ||
        data.sender.hasTag("admin") ||
        data.sender.hasTag("Staff") ||
        data.sender.hasTag("staff") ||
        data.sender.hasTag("Owner") ||
        data.sender.hasTag("owner")
    ) {
        await LegendaryAdmin_Main_Menu(data.sender);
        return;
    }

    function isOnCooldown(player) {
        const now = Date.now();
        if (!Legendarycooldowns.has(player.name)) return false;
        
        const playerCooldown = Legendarycooldowns.get(player.name);
        return (now - playerCooldown) < cooldownTime;
    }
    
    if (isOnCooldown(data.sender)) {
        const remainingTime = cooldownTime - (Date.now() - Legendarycooldowns.get(data.sender.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.sender.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the legendary crate again.✨`);
        data.sender.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await data.sender.playSound('note.pling', { volume: 1 });


    const crateMenu = new ActionFormData()
        .title('!Legendary Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.sender, crateMenu);
        result.selection === 0 ? await LegendaryCrate_Form(data.sender) : await LegendarycloseCrateMenu(data.sender);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});

//  ██████╗██████╗  █████╗ ████████╗███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ██║     ██████╔╝███████║   ██║   █████╗      ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                   
async function LegendaryCrate_Form(player) {
    const crateTimerValue = LegendarySettingsDb.get('LegendaryKittysCrateSettings:LegendaryCrateTimer') || '2';
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds

    function isOnCooldown(player) {
        const now = Date.now();
        if (!Legendarycooldowns.has(player.name)) return false;
        const cooldown = Legendarycooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }
     if (
        player.hasTag("Admin") ||
        player.hasTag("admin") ||
        player.hasTag("Staff") ||
        player.hasTag("staff") ||
        player.hasTag("Owner") ||
        player.hasTag("owner")
    ) {
        const allPrizes = getLegendaryPrizes();
        const randomPrize = getLegendary_RandomPrize(allPrizes);
        await givePlayerLegendary_Prize(player, randomPrize);
        await Legendary_Crate_Result_Form(player, randomPrize, allPrizes);
        return;
    }
    if (isOnCooldown(player)) {
        const remainingTime = cooldownTime - (Date.now() - Legendarycooldowns.get(player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the legendary crate again.✨`);
        player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    const allPrizes = getLegendaryPrizes();
    const randomPrize = getLegendary_RandomPrize(allPrizes);
    await givePlayerLegendary_Prize(player, randomPrize);
    await Legendary_Crate_Result_Form(player, randomPrize, allPrizes);

    Legendarycooldowns.set(player.name, Date.now());
}

const getLegendaryPrizes = () => {
    const customPrizes = LegendaryItemsDb.getAll();
    if (customPrizes.length === 0) {
        return Legendaryprizes;
    }
    return customPrizes.map(itemKey => {
        const itemData = JSON.parse(LegendaryItemsDb.get(itemKey));
        return {
            item: itemKey,
            quantity: itemData.itemValue || 1,
            weight: itemData.weight || 1,
        };
    });
};

function getLegendary_RandomPrize(Legendaryprizes) {
    const totalWeight = Legendaryprizes.reduce((sum, prize) => sum + prize.weight, 0);
    const randomNum = Math.floor(Math.random() * totalWeight);

    let runningSum = 0;
    for (const prize of Legendaryprizes) {
        runningSum += prize.weight;
        if (randomNum < runningSum) return prize;
    }
}

async function givePlayerLegendary_Prize(player, prize) {
    const overworld = world.getDimension('overworld');
    await overworld.runCommandAsync(`give "${player.name}" ${prize.item} ${prize.quantity}`);
    LegendarysetCooldown(player.name);
    player.playSound('kittys.lcrate.open', { volume: 1 });
}

//  ██████╗██████╗  █████╗ ████████╗███████╗    ██████╗ ███████╗███████╗██╗   ██╗██╗  ████████╗    ███████╗ ██████╗ ██████╗ ███╗   ███╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔══██╗██╔════╝██╔════╝██║   ██║██║  ╚══██╔══╝    ██╔════╝██╔═══██╗██╔══██╗████╗ ████║
// ██║     ██████╔╝███████║   ██║   █████╗      ██████╔╝█████╗  ███████╗██║   ██║██║     ██║       █████╗  ██║   ██║██████╔╝██╔████╔██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██╔══██╗██╔══╝  ╚════██║██║   ██║██║     ██║       ██╔══╝  ██║   ██║██╔══██╗██║╚██╔╝██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║  ██║███████╗███████║╚██████╔╝███████╗██║       ██║     ╚██████╔╝██║  ██║██║ ╚═╝ ██║
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝  ╚═╝╚══════╝╚══════╝ ╚═════╝ ╚══════╝╚═╝       ╚═╝      ╚═════╝ ╚═╝  ╚═╝╚═╝     ╚═╝                                                                                                                                                                                                                                                             
async function Legendary_Crate_Result_Form(player, wonPrize, LegendaryallPrizes) {
    const spinResultForm = new ActionFormData()
        .title(' !Legendary Crate Opened!')

    let texturePath;
    if (isitem.has(wonPrize.item)) {
        texturePath = `textures/items/${wonPrize.item}`;
    } else if (isblock.has(wonPrize.item)) {
        texturePath = `textures/blocks/${wonPrize.item}`;
    } else {
        texturePath = `textures/ui/gift_square`;
    }
    spinResultForm.button(`You Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''}!✨`, texturePath);

    const ToggleLegendaryCrateAnnouncementcommand = LegendarySettingsDb.get('LegendaryKittysCrateSettings:ToggleLegendaryCrateAnnouncementcommand') || false;
    if (ToggleLegendaryCrateAnnouncementcommand === 'false' || ToggleLegendaryCrateAnnouncementcommand === false) {
        world.sendMessage(`${player.name} Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''} From The Legendary Crate!✨`);
    }
    LegendaryallPrizes
        .filter(prize => prize !== wonPrize)
        .forEach(prize => {
            let texturePath;
            if (isitem.has(prize.item)) {
                texturePath = `textures/items/${prize.item}`;
            } else if (isblock.has(prize.item)) {
                texturePath = `textures/blocks/${prize.item}`;
            } else {
                texturePath = `textures/ui/gift_square`;
            }

            spinResultForm.button(`${prize.quantity} ${prize.item.replace(/_/g, ' ')}${prize.quantity > 1 ? 's' : ''}✨`, texturePath);
        });

    spinResultForm.button('Close');

    await spinResultForm.show(player);
    try {
        await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
        await player.playSound('note.bass', { volume: 0.5 });
    } catch (error) {
        console.error(`Error displaying spin result: ${error.message}`);
    }
}

const getLegendarySettingsDb = () => {
    const savedSettings = {
        LegendaryCrateTimer: LegendarySettingsDb.get('LegendaryKittysCrateSettings:LegendaryCrateTimer') || '2',
        LegendaryCrate_Command: LegendarySettingsDb.get('LegendaryKittysCrateSettings:LegendaryCrate_Command') || '!legendarycrate',
        ToggleLegendaryCrateChatcommand: LegendarySettingsDb.get('LegendaryKittysCrateSettings:ToggleLegendaryCrateChatcommand') || false,
        ToggleLegendaryCrateAnnouncementcommand: LegendarySettingsDb.get('LegendaryKittysCrateSettings:ToggleLegendaryCrateAnnouncementcommand') || false,
    };
    return savedSettings;
};

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝ 
async function LegendaryAdmin_Main_Menu(sender) {
    await sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await sender.playSound('note.pling', { volume: 1 });

    const crateMenu = new ActionFormData()
        .title('Admin !Legendary Crate')
        .body('Admin !Legendary Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel')
        .button('Settings', 'textures/ui/gear');

    try {
        const result = await forceShow(sender, crateMenu);
        if (result.selection === 0) {
            await LegendaryCrate_Form(sender);
        } else if (result.selection === 2) {
            await LegendaryAdmin_Crate_Settings_Main(sender);
        } else {
            await LegendarycloseCrateMenu(sender);
        }
    } catch (error) {
        console.error(`Error displaying crate menu: ${error.message}`);
    }
}

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗     ██████╗██████╗  █████╗ ████████╗███████╗    ███████╗███████╗████████╗████████╗██╗███╗   ██╗ ██████╗ ███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔════╝██╔════╝╚══██╔══╝╚══██╔══╝██║████╗  ██║██╔════╝ ██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██║     ██████╔╝███████║   ██║   █████╗      ███████╗█████╗     ██║      ██║   ██║██╔██╗ ██║██║  ███╗███████╗    ██╔████╔██║███████║██║██╔██╗ ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ╚════██║██╔══╝     ██║      ██║   ██║██║╚██╗██║██║   ██║╚════██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ███████║███████╗   ██║      ██║   ██║██║ ╚████║╚██████╔╝███████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝     ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚══════╝╚══════╝   ╚═╝      ╚═╝   ╚═╝╚═╝  ╚═══╝ ╚═════╝ ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝                                                                                                                                                                                      
async function LegendaryAdmin_Crate_Settings_Main(player) {
    const customUi = new ActionFormData()
        .title("Kittys Legendary Crate Settings Main Menu")
        .body("")
        .button("Crate Settings", "textures/ui/accessibility_glyph_color")
        .button("Add Item", "textures/ui/icon_best3")
        .button("Remove Item", "textures/ui/enable_editor");

    customUi.show(player).then(response => {

        switch (response.selection) {

            case 0:
                const LegendarysavedSettings = getLegendarySettingsDb();
                const LegendaryCrateTimerValue = Number(LegendarysavedSettings.LegendaryCrateTimer);
                const ToggleLegendaryCrateChatcommandValue = LegendarysavedSettings.ToggleLegendaryCrateChatcommand === 'true' ? true : false;
                const ToggleLegendaryCrateAnnouncementcommandValue = LegendarysavedSettings.ToggleLegendaryCrateAnnouncementcommand === 'true' ? true : false;

                const form = new ModalFormData()
                    .title('Kittys Crate Legendary Settings Menu')
                    .slider(`Crate Reset Time (Hours)✨`, 1, 24, 1, LegendaryCrateTimerValue)
                    .textField('Set The Crate Command,✨\nThis Will Be The New Command To Open The Crate Menu', '!legendarycrate', LegendarysavedSettings.LegendaryCrate_Command)
                    .toggle(`Disable ${LegendarysavedSettings.LegendaryCrate_Command} Chat Command`, ToggleLegendaryCrateChatcommandValue)
                    .toggle(`Disable !legendarycrate Chat Prize Announcement`, ToggleLegendaryCrateAnnouncementcommandValue);

                form.show(player).then(response => {
                    const [LegendaryCrateTimer, LegendaryCrate_Command, ToggleLegendaryCrateChatcommand, ToggleLegendaryCrateAnnouncementcommand] = response.formValues;
                    // Validate Crate_Command to ensure it's a non-empty string
                    if (!LegendaryCrate_Command || typeof LegendaryCrate_Command !== 'string') {
                        world.sendMessage("⚠️ Please provide a valid crate command.");
                        player.playSound('note.bass', { volume: 0.5 });
                        return LegendaryAdmin_Main_Menu(player);
                    }
                    LegendarySettingsDb.set('LegendaryKittysCrateSettings:ToggleLegendaryCrateChatcommand', `${ToggleLegendaryCrateChatcommand}`);
                    LegendarySettingsDb.set('LegendaryKittysCrateSettings:ToggleLegendaryCrateAnnouncementcommand', `${ToggleLegendaryCrateAnnouncementcommand}`);
                    LegendarySettingsDb.set('LegendaryKittysCrateSettings:LegendaryCrateTimer', `${LegendaryCrateTimer}`);
                    LegendarySettingsDb.set('LegendaryKittysCrateSettings:LegendaryCrate_Command', LegendaryCrate_Command);
                    return LegendaryAdmin_Main_Menu(player);
                });
                break;

            //  █████╗ ██████╗ ██████╗     ██╗████████╗███████╗███╗   ███╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔══██╗██╔══██╗    ██║╚══██╔══╝██╔════╝████╗ ████║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ███████║██║  ██║██║  ██║    ██║   ██║   █████╗  ██╔████╔██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██║██║  ██║██║  ██║    ██║   ██║   ██╔══╝  ██║╚██╔╝██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║██████╔╝██████╔╝    ██║   ██║   ███████╗██║ ╚═╝ ██║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚═════╝ ╚═════╝     ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                     
            case 1:
                (async () => {
                    const form = new ModalFormData()
                        .title('Kittys Legendary Add Item Menu')
                        .textField('Add Item\nThis Will Display On The Crate Prize Screen:✨', 'apple, iron_sword')
                        .slider(`Quantity✨`, 1, 64, 1, 1)
                        .slider(`0 = Rare\n100 = Common\nRarity✨`, 1, 100, 1, 1);

                    const response = await form.show(player);
                    try {
                        const itemName = response.formValues[0];
                        if (!itemName) {
                            world.sendMessage("Please enter a valid item name.✨");
                            player.playSound('note.bass', { volume: 0.5 });
                            return;
                        }

                        const itemData = {
                            itemValue: response.formValues[1], // Quantity
                            rarity: response.formValues[2] // Rarity
                        };

                        LegendaryItemsDb.set(`${itemName}`, JSON.stringify(itemData));
                        world.sendMessage(`Item added: ${itemName} (Quantity: ${itemData.itemValue}, Rarity: ${itemData.rarity})✨`);
                        player.playSound('random.levelup', { volume: 0.8 });
                        return LegendaryAdmin_Main_Menu(player);

                    } catch (error) {
                        world.sendMessage(`Failed to add item: ${error.message}✨`);
                        player.playSound('note.bass', { volume: 0.5 });
                    }
                })();
                break;

            // ██████╗ ███████╗███╗   ███╗ ██████╗ ██╗   ██╗███████╗    ██╗████████╗███████╗███╗   ███╗███████╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔════╝████╗ ████║██╔═══██╗██║   ██║██╔════╝    ██║╚══██╔══╝██╔════╝████╗ ████║██╔════╝    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ██████╔╝█████╗  ██╔████╔██║██║   ██║██║   ██║█████╗      ██║   ██║   █████╗  ██╔████╔██║███████╗    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██╗██╔══╝  ██║╚██╔╝██║██║   ██║╚██╗ ██╔╝██╔══╝      ██║   ██║   ██╔══╝  ██║╚██╔╝██║╚════██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║███████╗██║ ╚═╝ ██║╚██████╔╝ ╚████╔╝ ███████╗    ██║   ██║   ███████╗██║ ╚═╝ ██║███████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚══════╝╚═╝     ╚═╝ ╚═════╝   ╚═══╝  ╚══════╝    ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝╚══════╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                                          
            case 2:
                (async () => {
                    const allLegendaryItemsDb = LegendaryItemsDb.getAll();
                    if (allLegendaryItemsDb.length === 0) {
                        world.sendMessage("No Items to remove.✨");
                        player.playSound('note.bass', { volume: 0.5 });
                        return;
                    }
                    let page = 0;
                    const totalPages = Math.ceil(allLegendaryItemsDb.length / 5);
                    while (true) {
                        const start = page * 5;
                        const end = start + 5;

                        const form = new ActionFormData()
                            .title("Kittys Legendary Remove Items Menu")
                            .body(`Select an Item to Remove:\nPage: ${page + 1} of ${totalPages}✨`);
                        allLegendaryItemsDb.slice(start, end).forEach((itemKey) => {
                            const itemData = JSON.parse(LegendaryItemsDb.get(itemKey));
                            const itemQuantity = itemData.itemValue;
                            const itemRarity = itemData.rarity; // Extract rarity

                            let texturePath;
                            if (isitem.has(itemKey)) {
                                texturePath = `textures/items/${itemKey}`;
                            } else if (isblock.has(itemKey)) {
                                texturePath = `textures/blocks/${itemKey}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            form.button(` §l§cRemove:\n §l§r${itemKey} (x${itemQuantity}) \n(Rarity: ${itemRarity})✨`, texturePath);
                        });
                        if (page > 0) {
                            form.button(` §o§1Last Page\n §3[ Page: ${page} ]✨`, "textures/ui/generic_dpad_left");
                        }
                        if (end < allLegendaryItemsDb.length) {
                            form.button(` §o§1Next Page\n §3[ Page: ${page + 2} ]✨`, "textures/ui/generic_dpad_right");
                        }
                        form.button("§l§cMain\nMenu✨", "textures/ui/icon_map");

                        const response = await form.show(player);
                        const selectedItemIndex = response.selection;
                        const totalButtonsOnPage = allLegendaryItemsDb.slice(start, end).length;
                        const hasPreviousButton = page > 0 ? 1 : 0;
                        const hasNextButton = end < allLegendaryItemsDb.length ? 1 : 0;
                        if (selectedItemIndex === totalButtonsOnPage + hasPreviousButton + hasNextButton) {
                            return LegendaryAdmin_Main_Menu(player);
                        }
                        if (page > 0 && selectedItemIndex === totalButtonsOnPage) {
                            page--;
                            continue;
                        }
                        if (end < allLegendaryItemsDb.length && selectedItemIndex === totalButtonsOnPage + hasPreviousButton) {
                            page++;
                            continue;
                        }
                        const itemToRemove = allLegendaryItemsDb[start + selectedItemIndex];
                        const itemData = JSON.parse(LegendaryItemsDb.get(itemToRemove));
                        const itemQuantity = itemData.itemValue;
                        const itemRarity = itemData.rarity;
                        try {
                            let texturePath;
                            if (isitem.has(itemToRemove)) {
                                texturePath = `textures/items/${itemToRemove}`;
                            } else if (isblock.has(itemToRemove)) {
                                texturePath = `textures/blocks/${itemToRemove}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            //  ██████╗ ██████╗ ███╗   ██╗███████╗██╗██████╗ ███╗   ███╗    ██╗████████╗███████╗███╗   ███╗    ██████╗ ███████╗███╗   ███╗ ██████╗ ██╗   ██╗ █████╗ ██╗         ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
                            // ██╔════╝██╔═══██╗████╗  ██║██╔════╝██║██╔══██╗████╗ ████║    ██║╚══██╔══╝██╔════╝████╗ ████║    ██╔══██╗██╔════╝████╗ ████║██╔═══██╗██║   ██║██╔══██╗██║         ████╗ ████║██╔════╝████╗  ██║██║   ██║
                            // ██║     ██║   ██║██╔██╗ ██║█████╗  ██║██████╔╝██╔████╔██║    ██║   ██║   █████╗  ██╔████╔██║    ██████╔╝█████╗  ██╔████╔██║██║   ██║██║   ██║███████║██║         ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
                            // ██║     ██║   ██║██║╚██╗██║██╔══╝  ██║██╔══██╗██║╚██╔╝██║    ██║   ██║   ██╔══╝  ██║╚██╔╝██║    ██╔══██╗██╔══╝  ██║╚██╔╝██║██║   ██║╚██╗ ██╔╝██╔══██║██║         ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
                            // ╚██████╗╚██████╔╝██║ ╚████║██║     ██║██║  ██║██║ ╚═╝ ██║    ██║   ██║   ███████╗██║ ╚═╝ ██║    ██║  ██║███████╗██║ ╚═╝ ██║╚██████╔╝ ╚████╔╝ ██║  ██║███████╗    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
                            //  ╚═════╝ ╚═════╝ ╚═╝  ╚═══╝╚═╝     ╚═╝╚═╝  ╚═╝╚═╝     ╚═╝    ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝    ╚═╝  ╚═╝╚══════╝╚═╝     ╚═╝ ╚═════╝   ╚═══╝  ╚═╝  ╚═╝╚══════╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝ 
                            const ConfirmForm = new ActionFormData()
                                .title('Legendary Confirm Item Removal')
                                .body(`Are you sure you want to remove the item: §l${itemToRemove} (x${itemQuantity}) (Rarity: ${itemRarity})§r?✨`)
                                .button("Go Back\n[ Clear Item ]✨", "textures/ui/icon_map")
                                .button(`Remove: §l${itemToRemove} (x${itemQuantity}) (Rarity: ${itemRarity})§r?✨`, texturePath)
                                .button("Go Back\n[ Clear Item ]✨", "textures/ui/icon_map");

                            const confirmResponse = await ConfirmForm.show(player);

                            if (confirmResponse.selection === 1) {
                                try {
                                    LegendaryItemsDb.del(itemToRemove);
                                    world.sendMessage(`Item removed: ${itemToRemove} (x${itemQuantity}) (Rarity: ${itemRarity})✨`);
                                    player.playSound('note.bass', { volume: 0.5 });
                                    return LegendaryAdmin_Main_Menu(player);
                                } catch (error) {
                                    console.warn(error);
                                    world.sendMessage("Error occurred while removing item.✨");
                                    player.playSound('note.bass', { volume: 0.5 });
                                }
                            }
                        } catch (error) {
                            world.sendMessage("Error occurred while showing the confirmation dialog.✨");
                            player.playSound('note.bass', { volume: 0.5 });
                        }
                    }
                })();
                break;
        }
    });
}
async function LegendarycloseCrateMenu(player) {
    await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
    await player.playSound('note.bass', { volume: 0.5 });
}

// ██╗    ██╗ ██████╗ ██████╗ ██╗     ██████╗     ███████╗██╗   ██╗███████╗███╗   ██╗████████╗███████╗
// ██║    ██║██╔═══██╗██╔══██╗██║     ██╔══██╗    ██╔════╝██║   ██║██╔════╝████╗  ██║╚══██╔══╝██╔════╝
// ██║ █╗ ██║██║   ██║██████╔╝██║     ██║  ██║    █████╗  ██║   ██║█████╗  ██╔██╗ ██║   ██║   ███████╗
// ██║███╗██║██║   ██║██╔══██╗██║     ██║  ██║    ██╔══╝  ╚██╗ ██╔╝██╔══╝  ██║╚██╗██║   ██║   ╚════██║
// ╚███╔███╔╝╚██████╔╝██║  ██║███████╗██████╔╝    ███████╗ ╚████╔╝ ███████╗██║ ╚████║   ██║   ███████║
//  ╚══╝╚══╝  ╚═════╝ ╚═╝  ╚═╝╚══════╝╚═════╝     ╚══════╝  ╚═══╝  ╚══════╝╚═╝  ╚═══╝   ╚═╝   ╚══════╝
const legendaryChests = new Set()

world.afterEvents.entitySpawn.subscribe((data) => {
    if (data.entity.typeId != "kittys:legendarychest") return
    const legendarychest = data.entity
    legendaryChests.add(legendarychest)
    system.runTimeout(() => {
        if (!legendarychest.isValid()) return
        legendarychest.nameTag = `§6Legendary Crate`
    }, 10)
})

world.beforeEvents.playerInteractWithEntity.subscribe(async (data) => {
    const { player, target } = data
    if (target.typeId != "kittys:legendarychest") return
    const LegendarycrateTimerValue = LegendarySettingsDb.get('KittysLegendaryCrateSettings:LegendaryCrateTimer') || '2';
    const LegendarycrateTimerHours = Number(LegendarycrateTimerValue);
    const cooldownTime = LegendarycrateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    data.cancel = true;
    system.run(() => {
        target.dimension.playSound("random.pop", target.location, { pitch: 0.5 })
    })

    if (
        data.player.hasTag("Admin") ||
        data.player.hasTag("admin") ||
        data.player.hasTag("Staff") ||
        data.player.hasTag("staff") ||
        data.player.hasTag("Owner") ||
        data.player.hasTag("owner")
    ) {
        await LegendaryAdmin_Main_Menu(data.player);
        return;
    }

    function isOnCooldown(player) {
        const now = Date.now();
        if (!Legendarycooldowns.has(player.name)) return false;
        const cooldown = Legendarycooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }

    if (isOnCooldown(data.player)) {
        const remainingTime = cooldownTime - (Date.now() - Legendarycooldowns.get(data.player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the legendary crate again.✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.player.sendMessage('§6§lOpening Crate Menu!✨');


    const LegendarycrateMenu = new ActionFormData()
        .title('!Legendary Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.player, LegendarycrateMenu);
        result.selection === 0 ? await LegendaryCrate_Form(data.player) : await LegendarycloseCrateMenu(data.player);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});
                                                                                                                 
//  ██████╗ ██████╗ ███╗   ██╗███████╗████████╗███████╗
// ██╔════╝██╔═══██╗████╗  ██║██╔════╝╚══██╔══╝██╔════╝
// ██║     ██║   ██║██╔██╗ ██║███████╗   ██║   ███████╗
// ██║     ██║   ██║██║╚██╗██║╚════██║   ██║   ╚════██║
// ╚██████╗╚██████╔╝██║ ╚████║███████║   ██║   ███████║
//  ╚═════╝ ╚═════╝ ╚═╝  ╚═══╝╚══════╝   ╚═╝   ╚══════╝
const GalaxySettingsDb = new Database("GalaxySettings");
const GalaxyItemsDb = new Database("GalaxyItems");
const Galaxycooldowns = new Map();
const Galaxyprizes = [
    { item: 'netherite_sword', quantity: 1, weight: 5 },     
    { item: 'elytra', quantity: 1, weight: 3 },             
    { item: 'totem_of_undying', quantity: 1, weight: 4 },    
    { item: 'phantom_membrane', quantity: 3, weight: 2 },   
    { item: 'end_crystal', quantity: 1, weight: 1 },         
    { item: 'netherite_block', quantity: 1, weight: 1 },   
    { item: 'respawn_anchor', quantity: 1, weight: 2 },      
    { item: 'dragon_head', quantity: 1, weight: 0.5 },      
    { item: 'music_disc_11', quantity: 1, weight: 3 },    
    { item: 'lodestone', quantity: 1, weight: 1 },           
];

function GalaxysetCooldown(player) {
    Galaxycooldowns.set(player, Date.now());
}

//  ██████╗██╗  ██╗ █████╗ ████████╗     ██████╗ ██████╗ ███╗   ███╗███╗   ███╗ █████╗ ███╗   ██╗██████╗ 
// ██╔════╝██║  ██║██╔══██╗╚══██╔══╝    ██╔════╝██╔═══██╗████╗ ████║████╗ ████║██╔══██╗████╗  ██║██╔══██╗
// ██║     ███████║███████║   ██║       ██║     ██║   ██║██╔████╔██║██╔████╔██║███████║██╔██╗ ██║██║  ██║
// ██║     ██╔══██║██╔══██║   ██║       ██║     ██║   ██║██║╚██╔╝██║██║╚██╔╝██║██╔══██║██║╚██╗██║██║  ██║
// ╚██████╗██║  ██║██║  ██║   ██║       ╚██████╗╚██████╔╝██║ ╚═╝ ██║██║ ╚═╝ ██║██║  ██║██║ ╚████║██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝        ╚═════╝ ╚═════╝ ╚═╝     ╚═╝╚═╝     ╚═╝╚═╝  ╚═╝╚═╝  ╚═══╝╚═════╝ 
system.runInterval(() => {
    const ToggleGalaxyCrateChatcommand = GalaxySettingsDb.get('GalaxyKittysCrateSettings:ToggleGalaxyCrateChatcommand') || false;
    if (ToggleGalaxyCrateChatcommand === 'true' || ToggleGalaxyCrateChatcommand === true) {
        return;
    }
    const GalaxyCrate_Command = GalaxySettingsDb.get('GalaxyKittysCrateSettings:GalaxyCrate_Command') || '!galaxycrate';
    world.getDimension('overworld').runCommand(`say Open the crate menu with §6${GalaxyCrate_Command}`);
}, 15000);

world.beforeEvents.chatSend.subscribe(async (data) => {
    const Crate_Command = GalaxySettingsDb.get('GalaxyKittysCrateSettings:GalaxyCrate_Command') || '!galaxycrate';
    const crateTimerValue = GalaxySettingsDb.get('GalaxyKittysCrateSettings:GalaxyCrateTimer') || '2';
    const ToggleGalaxyCrateChatcommand = GalaxySettingsDb.get('GalaxyKittysCrateSettings:ToggleGalaxyCrateChatcommand') || false;
    if (ToggleGalaxyCrateChatcommand === 'true' || ToggleGalaxyCrateChatcommand === true) {
        return;
    }
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    if (!data.message.startsWith(Crate_Command)) return;
    data.cancel = true;

    if (
        data.sender.hasTag("Admin") ||
        data.sender.hasTag("admin") ||
        data.sender.hasTag("Staff") ||
        data.sender.hasTag("staff") ||
        data.sender.hasTag("Owner") ||
        data.sender.hasTag("owner")
    ) {
        await GalaxyAdmin_Main_Menu(data.sender);
        return;
    }

    function isOnCooldown(player) {
        const now = Date.now();
        if (!Galaxycooldowns.has(player.name)) return false;
        
        const playerCooldown = Galaxycooldowns.get(player.name);
        return (now - playerCooldown) < cooldownTime;
    }
    
    if (isOnCooldown(data.sender)) {
        const remainingTime = cooldownTime - (Date.now() - Galaxycooldowns.get(data.sender.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.sender.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the galaxy crate again.✨`);
        data.sender.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await data.sender.playSound('note.pling', { volume: 1 });


    const crateMenu = new ActionFormData()
        .title('!Galaxy Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.sender, crateMenu);
        result.selection === 0 ? await GalaxyCrate_Form(data.sender) : await GalaxycloseCrateMenu(data.sender);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});

//  ██████╗██████╗  █████╗ ████████╗███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ██║     ██████╔╝███████║   ██║   █████╗      ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                   
async function GalaxyCrate_Form(player) {
    const crateTimerValue = GalaxySettingsDb.get('GalaxyKittysCrateSettings:GalaxyCrateTimer') || '2';
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds

    function isOnCooldown(player) {
        const now = Date.now();
        if (!Galaxycooldowns.has(player.name)) return false;
        const cooldown = Galaxycooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }
     if (
        player.hasTag("Admin") ||
        player.hasTag("admin") ||
        player.hasTag("Staff") ||
        player.hasTag("staff") ||
        player.hasTag("Owner") ||
        player.hasTag("owner")
    ) {
        const allPrizes = getGalaxyPrizes();
        const randomPrize = getGalaxy_RandomPrize(allPrizes);
        await givePlayerGalaxy_Prize(player, randomPrize);
        await Galaxy_Crate_Result_Form(player, randomPrize, allPrizes);
        return;
    }
    if (isOnCooldown(player)) {
        const remainingTime = cooldownTime - (Date.now() - Galaxycooldowns.get(player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the galaxy crate again.✨`);
        player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    const allPrizes = getGalaxyPrizes();
    const randomPrize = getGalaxy_RandomPrize(allPrizes);
    await givePlayerGalaxy_Prize(player, randomPrize);
    await Galaxy_Crate_Result_Form(player, randomPrize, allPrizes);

    Galaxycooldowns.set(player.name, Date.now());
}

const getGalaxyPrizes = () => {
    const customPrizes = GalaxyItemsDb.getAll();
    if (customPrizes.length === 0) {
        return Galaxyprizes;
    }
    return customPrizes.map(itemKey => {
        const itemData = JSON.parse(GalaxyItemsDb.get(itemKey));
        return {
            item: itemKey,
            quantity: itemData.itemValue || 1,
            weight: itemData.weight || 1,
        };
    });
};

function getGalaxy_RandomPrize(Galaxyprizes) {
    const totalWeight = Galaxyprizes.reduce((sum, prize) => sum + prize.weight, 0);
    const randomNum = Math.floor(Math.random() * totalWeight);

    let runningSum = 0;
    for (const prize of Galaxyprizes) {
        runningSum += prize.weight;
        if (randomNum < runningSum) return prize;
    }
}

async function givePlayerGalaxy_Prize(player, prize) {
    const overworld = world.getDimension('overworld');
    await overworld.runCommandAsync(`give "${player.name}" ${prize.item} ${prize.quantity}`);
    GalaxysetCooldown(player.name);
    player.playSound('kittys.lcrate.open', { volume: 1 });
}

//  ██████╗██████╗  █████╗ ████████╗███████╗    ██████╗ ███████╗███████╗██╗   ██╗██╗  ████████╗    ███████╗ ██████╗ ██████╗ ███╗   ███╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔══██╗██╔════╝██╔════╝██║   ██║██║  ╚══██╔══╝    ██╔════╝██╔═══██╗██╔══██╗████╗ ████║
// ██║     ██████╔╝███████║   ██║   █████╗      ██████╔╝█████╗  ███████╗██║   ██║██║     ██║       █████╗  ██║   ██║██████╔╝██╔████╔██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██╔══██╗██╔══╝  ╚════██║██║   ██║██║     ██║       ██╔══╝  ██║   ██║██╔══██╗██║╚██╔╝██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║  ██║███████╗███████║╚██████╔╝███████╗██║       ██║     ╚██████╔╝██║  ██║██║ ╚═╝ ██║
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝  ╚═╝╚══════╝╚══════╝ ╚═════╝ ╚══════╝╚═╝       ╚═╝      ╚═════╝ ╚═╝  ╚═╝╚═╝     ╚═╝                                                                                                                                                                                                                                                             
async function Galaxy_Crate_Result_Form(player, wonPrize, GalaxyallPrizes) {
    const spinResultForm = new ActionFormData()
        .title(' !Galaxy Crate Opened!')

    let texturePath;
    if (isitem.has(wonPrize.item)) {
        texturePath = `textures/items/${wonPrize.item}`;
    } else if (isblock.has(wonPrize.item)) {
        texturePath = `textures/blocks/${wonPrize.item}`;
    } else {
        texturePath = `textures/ui/gift_square`;
    }
    spinResultForm.button(`You Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''}!✨`, texturePath);

    const ToggleGalaxyCrateAnnouncementcommand = GalaxySettingsDb.get('GalaxyKittysCrateSettings:ToggleGalaxyCrateAnnouncementcommand') || false;
    if (ToggleGalaxyCrateAnnouncementcommand === 'false' || ToggleGalaxyCrateAnnouncementcommand === false) {
        world.sendMessage(`${player.name} Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''} From The Galaxy Crate!✨`);
    }
    GalaxyallPrizes
        .filter(prize => prize !== wonPrize)
        .forEach(prize => {
            let texturePath;
            if (isitem.has(prize.item)) {
                texturePath = `textures/items/${prize.item}`;
            } else if (isblock.has(prize.item)) {
                texturePath = `textures/blocks/${prize.item}`;
            } else {
                texturePath = `textures/ui/gift_square`;
            }

            spinResultForm.button(`${prize.quantity} ${prize.item.replace(/_/g, ' ')}${prize.quantity > 1 ? 's' : ''}✨`, texturePath);
        });

    spinResultForm.button('Close');

    await spinResultForm.show(player);
    try {
        await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
        await player.playSound('note.bass', { volume: 0.5 });
    } catch (error) {
        console.error(`Error displaying spin result: ${error.message}`);
    }
}

const getGalaxySettingsDb = () => {
    const savedSettings = {
        GalaxyCrateTimer: GalaxySettingsDb.get('GalaxyKittysCrateSettings:GalaxyCrateTimer') || '2',
        GalaxyCrate_Command: GalaxySettingsDb.get('GalaxyKittysCrateSettings:GalaxyCrate_Command') || '!galaxycrate',
        ToggleGalaxyCrateChatcommand: GalaxySettingsDb.get('GalaxyKittysCrateSettings:ToggleGalaxyCrateChatcommand') || false,
        ToggleGalaxyCrateAnnouncementcommand: GalaxySettingsDb.get('GalaxyKittysCrateSettings:ToggleGalaxyCrateAnnouncementcommand') || false,
    };
    return savedSettings;
};

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝ 
async function GalaxyAdmin_Main_Menu(sender) {
    await sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await sender.playSound('note.pling', { volume: 1 });

    const crateMenu = new ActionFormData()
        .title('Admin !Galaxy Crate')
        .body('Admin !Galaxy Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel')
        .button('Settings', 'textures/ui/gear');

    try {
        const result = await forceShow(sender, crateMenu);
        if (result.selection === 0) {
            await GalaxyCrate_Form(sender);
        } else if (result.selection === 2) {
            await GalaxyAdmin_Crate_Settings_Main(sender);
        } else {
            await GalaxycloseCrateMenu(sender);
        }
    } catch (error) {
        console.error(`Error displaying crate menu: ${error.message}`);
    }
}

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗     ██████╗██████╗  █████╗ ████████╗███████╗    ███████╗███████╗████████╗████████╗██╗███╗   ██╗ ██████╗ ███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔════╝██╔════╝╚══██╔══╝╚══██╔══╝██║████╗  ██║██╔════╝ ██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██║     ██████╔╝███████║   ██║   █████╗      ███████╗█████╗     ██║      ██║   ██║██╔██╗ ██║██║  ███╗███████╗    ██╔████╔██║███████║██║██╔██╗ ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ╚════██║██╔══╝     ██║      ██║   ██║██║╚██╗██║██║   ██║╚════██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ███████║███████╗   ██║      ██║   ██║██║ ╚████║╚██████╔╝███████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝     ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚══════╝╚══════╝   ╚═╝      ╚═╝   ╚═╝╚═╝  ╚═══╝ ╚═════╝ ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝                                                                                                                                                                                      
async function GalaxyAdmin_Crate_Settings_Main(player) {
    const customUi = new ActionFormData()
        .title("Kittys Galaxy Crate Settings Main Menu")
        .body("")
        .button("Crate Settings", "textures/ui/accessibility_glyph_color")
        .button("Add Item", "textures/ui/icon_best3")
        .button("Remove Item", "textures/ui/enable_editor");

    customUi.show(player).then(response => {

        switch (response.selection) {

            case 0:
                const GalaxysavedSettings = getGalaxySettingsDb();
                const GalaxyCrateTimerValue = Number(GalaxysavedSettings.GalaxyCrateTimer);
                const ToggleGalaxyCrateChatcommandValue = GalaxysavedSettings.ToggleGalaxyCrateChatcommand === 'true' ? true : false;
                const ToggleGalaxyCrateAnnouncementcommandValue = GalaxysavedSettings.ToggleGalaxyCrateAnnouncementcommand === 'true' ? true : false;

                const form = new ModalFormData()
                    .title('Kittys Crate Galaxy Settings Menu')
                    .slider(`Crate Reset Time (Hours)✨`, 1, 24, 1, GalaxyCrateTimerValue)
                    .textField('Set The Crate Command,✨\nThis Will Be The New Command To Open The Crate Menu', '!galaxycrate', GalaxysavedSettings.GalaxyCrate_Command)
                    .toggle(`Disable ${GalaxysavedSettings.GalaxyCrate_Command} Chat Command`, ToggleGalaxyCrateChatcommandValue)
                    .toggle(`Disable !galaxycrate Chat Prize Announcement`, ToggleGalaxyCrateAnnouncementcommandValue);

                form.show(player).then(response => {
                    const [GalaxyCrateTimer, GalaxyCrate_Command, ToggleGalaxyCrateChatcommand, ToggleGalaxyCrateAnnouncementcommand] = response.formValues;
                    // Validate Crate_Command to ensure it's a non-empty string
                    if (!GalaxyCrate_Command || typeof GalaxyCrate_Command !== 'string') {
                        world.sendMessage("⚠️ Please provide a valid crate command.");
                        player.playSound('note.bass', { volume: 0.5 });
                        return GalaxyAdmin_Main_Menu(player);
                    }
                    GalaxySettingsDb.set('GalaxyKittysCrateSettings:ToggleGalaxyCrateChatcommand', `${ToggleGalaxyCrateChatcommand}`);
                    GalaxySettingsDb.set('GalaxyKittysCrateSettings:ToggleGalaxyCrateAnnouncementcommand', `${ToggleGalaxyCrateAnnouncementcommand}`);
                    GalaxySettingsDb.set('GalaxyKittysCrateSettings:GalaxyCrateTimer', `${GalaxyCrateTimer}`);
                    GalaxySettingsDb.set('GalaxyKittysCrateSettings:GalaxyCrate_Command', GalaxyCrate_Command);
                    return GalaxyAdmin_Main_Menu(player);
                });
                break;

            //  █████╗ ██████╗ ██████╗     ██╗████████╗███████╗███╗   ███╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔══██╗██╔══██╗    ██║╚══██╔══╝██╔════╝████╗ ████║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ███████║██║  ██║██║  ██║    ██║   ██║   █████╗  ██╔████╔██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██║██║  ██║██║  ██║    ██║   ██║   ██╔══╝  ██║╚██╔╝██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║██████╔╝██████╔╝    ██║   ██║   ███████╗██║ ╚═╝ ██║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚═════╝ ╚═════╝     ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                     
            case 1:
                (async () => {
                    const form = new ModalFormData()
                        .title('Kittys Galaxy Add Item Menu')
                        .textField('Add Item\nThis Will Display On The Crate Prize Screen:✨', 'apple, iron_sword')
                        .slider(`Quantity✨`, 1, 64, 1, 1)
                        .slider(`0 = Rare\n100 = Common\nRarity✨`, 1, 100, 1, 1);

                    const response = await form.show(player);
                    try {
                        const itemName = response.formValues[0];
                        if (!itemName) {
                            world.sendMessage("Please enter a valid item name.✨");
                            player.playSound('note.bass', { volume: 0.5 });
                            return;
                        }

                        const itemData = {
                            itemValue: response.formValues[1], // Quantity
                            rarity: response.formValues[2] // Rarity
                        };

                        GalaxyItemsDb.set(`${itemName}`, JSON.stringify(itemData));
                        world.sendMessage(`Item added: ${itemName} (Quantity: ${itemData.itemValue}, Rarity: ${itemData.rarity})✨`);
                        player.playSound('random.levelup', { volume: 0.8 });
                        return GalaxyAdmin_Main_Menu(player);

                    } catch (error) {
                        world.sendMessage(`Failed to add item: ${error.message}✨`);
                        player.playSound('note.bass', { volume: 0.5 });
                    }
                })();
                break;

            // ██████╗ ███████╗███╗   ███╗ ██████╗ ██╗   ██╗███████╗    ██╗████████╗███████╗███╗   ███╗███████╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔════╝████╗ ████║██╔═══██╗██║   ██║██╔════╝    ██║╚══██╔══╝██╔════╝████╗ ████║██╔════╝    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ██████╔╝█████╗  ██╔████╔██║██║   ██║██║   ██║█████╗      ██║   ██║   █████╗  ██╔████╔██║███████╗    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██╗██╔══╝  ██║╚██╔╝██║██║   ██║╚██╗ ██╔╝██╔══╝      ██║   ██║   ██╔══╝  ██║╚██╔╝██║╚════██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║███████╗██║ ╚═╝ ██║╚██████╔╝ ╚████╔╝ ███████╗    ██║   ██║   ███████╗██║ ╚═╝ ██║███████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚══════╝╚═╝     ╚═╝ ╚═════╝   ╚═══╝  ╚══════╝    ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝╚══════╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                                          
            case 2:
                (async () => {
                    const allGalaxyItemsDb = GalaxyItemsDb.getAll();
                    if (allGalaxyItemsDb.length === 0) {
                        world.sendMessage("No Items to remove.✨");
                        player.playSound('note.bass', { volume: 0.5 });
                        return;
                    }
                    let page = 0;
                    const totalPages = Math.ceil(allGalaxyItemsDb.length / 5);
                    while (true) {
                        const start = page * 5;
                        const end = start + 5;

                        const form = new ActionFormData()
                            .title("Kittys Galaxy Remove Items Menu")
                            .body(`Select an Item to Remove:\nPage: ${page + 1} of ${totalPages}✨`);
                        allGalaxyItemsDb.slice(start, end).forEach((itemKey) => {
                            const itemData = JSON.parse(GalaxyItemsDb.get(itemKey));
                            const itemQuantity = itemData.itemValue;
                            const itemRarity = itemData.rarity; // Extract rarity

                            let texturePath;
                            if (isitem.has(itemKey)) {
                                texturePath = `textures/items/${itemKey}`;
                            } else if (isblock.has(itemKey)) {
                                texturePath = `textures/blocks/${itemKey}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            form.button(` §l§cRemove:\n §l§r${itemKey} (x${itemQuantity}) \n(Rarity: ${itemRarity})✨`, texturePath);
                        });
                        if (page > 0) {
                            form.button(` §o§1Last Page\n §3[ Page: ${page} ]✨`, "textures/ui/generic_dpad_left");
                        }
                        if (end < allGalaxyItemsDb.length) {
                            form.button(` §o§1Next Page\n §3[ Page: ${page + 2} ]✨`, "textures/ui/generic_dpad_right");
                        }
                        form.button("§l§cMain\nMenu✨", "textures/ui/icon_map");

                        const response = await form.show(player);
                        const selectedItemIndex = response.selection;
                        const totalButtonsOnPage = allGalaxyItemsDb.slice(start, end).length;
                        const hasPreviousButton = page > 0 ? 1 : 0;
                        const hasNextButton = end < allGalaxyItemsDb.length ? 1 : 0;
                        if (selectedItemIndex === totalButtonsOnPage + hasPreviousButton + hasNextButton) {
                            return GalaxyAdmin_Main_Menu(player);
                        }
                        if (page > 0 && selectedItemIndex === totalButtonsOnPage) {
                            page--;
                            continue;
                        }
                        if (end < allGalaxyItemsDb.length && selectedItemIndex === totalButtonsOnPage + hasPreviousButton) {
                            page++;
                            continue;
                        }
                        const itemToRemove = allGalaxyItemsDb[start + selectedItemIndex];
                        const itemData = JSON.parse(GalaxyItemsDb.get(itemToRemove));
                        const itemQuantity = itemData.itemValue;
                        const itemRarity = itemData.rarity;
                        try {
                            let texturePath;
                            if (isitem.has(itemToRemove)) {
                                texturePath = `textures/items/${itemToRemove}`;
                            } else if (isblock.has(itemToRemove)) {
                                texturePath = `textures/blocks/${itemToRemove}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            //  ██████╗ ██████╗ ███╗   ██╗███████╗██╗██████╗ ███╗   ███╗    ██╗████████╗███████╗███╗   ███╗    ██████╗ ███████╗███╗   ███╗ ██████╗ ██╗   ██╗ █████╗ ██╗         ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
                            // ██╔════╝██╔═══██╗████╗  ██║██╔════╝██║██╔══██╗████╗ ████║    ██║╚══██╔══╝██╔════╝████╗ ████║    ██╔══██╗██╔════╝████╗ ████║██╔═══██╗██║   ██║██╔══██╗██║         ████╗ ████║██╔════╝████╗  ██║██║   ██║
                            // ██║     ██║   ██║██╔██╗ ██║█████╗  ██║██████╔╝██╔████╔██║    ██║   ██║   █████╗  ██╔████╔██║    ██████╔╝█████╗  ██╔████╔██║██║   ██║██║   ██║███████║██║         ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
                            // ██║     ██║   ██║██║╚██╗██║██╔══╝  ██║██╔══██╗██║╚██╔╝██║    ██║   ██║   ██╔══╝  ██║╚██╔╝██║    ██╔══██╗██╔══╝  ██║╚██╔╝██║██║   ██║╚██╗ ██╔╝██╔══██║██║         ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
                            // ╚██████╗╚██████╔╝██║ ╚████║██║     ██║██║  ██║██║ ╚═╝ ██║    ██║   ██║   ███████╗██║ ╚═╝ ██║    ██║  ██║███████╗██║ ╚═╝ ██║╚██████╔╝ ╚████╔╝ ██║  ██║███████╗    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
                            //  ╚═════╝ ╚═════╝ ╚═╝  ╚═══╝╚═╝     ╚═╝╚═╝  ╚═╝╚═╝     ╚═╝    ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝    ╚═╝  ╚═╝╚══════╝╚═╝     ╚═╝ ╚═════╝   ╚═══╝  ╚═╝  ╚═╝╚══════╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝ 
                            const ConfirmForm = new ActionFormData()
                                .title('Galaxy Confirm Item Removal')
                                .body(`Are you sure you want to remove the item: §l${itemToRemove} (x${itemQuantity}) (Rarity: ${itemRarity})§r?✨`)
                                .button("Go Back\n[ Clear Item ]✨", "textures/ui/icon_map")
                                .button(`Remove: §l${itemToRemove} (x${itemQuantity}) (Rarity: ${itemRarity})§r?✨`, texturePath)
                                .button("Go Back\n[ Clear Item ]✨", "textures/ui/icon_map");

                            const confirmResponse = await ConfirmForm.show(player);

                            if (confirmResponse.selection === 1) {
                                try {
                                    GalaxyItemsDb.del(itemToRemove);
                                    world.sendMessage(`Item removed: ${itemToRemove} (x${itemQuantity}) (Rarity: ${itemRarity})✨`);
                                    player.playSound('note.bass', { volume: 0.5 });
                                    return GalaxyAdmin_Main_Menu(player);
                                } catch (error) {
                                    console.warn(error);
                                    world.sendMessage("Error occurred while removing item.✨");
                                    player.playSound('note.bass', { volume: 0.5 });
                                }
                            }
                        } catch (error) {
                            world.sendMessage("Error occurred while showing the confirmation dialog.✨");
                            player.playSound('note.bass', { volume: 0.5 });
                        }
                    }
                })();
                break;
        }
    });
}
async function GalaxycloseCrateMenu(player) {
    await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
    await player.playSound('note.bass', { volume: 0.5 });
}

// ██╗    ██╗ ██████╗ ██████╗ ██╗     ██████╗     ███████╗██╗   ██╗███████╗███╗   ██╗████████╗███████╗
// ██║    ██║██╔═══██╗██╔══██╗██║     ██╔══██╗    ██╔════╝██║   ██║██╔════╝████╗  ██║╚══██╔══╝██╔════╝
// ██║ █╗ ██║██║   ██║██████╔╝██║     ██║  ██║    █████╗  ██║   ██║█████╗  ██╔██╗ ██║   ██║   ███████╗
// ██║███╗██║██║   ██║██╔══██╗██║     ██║  ██║    ██╔══╝  ╚██╗ ██╔╝██╔══╝  ██║╚██╗██║   ██║   ╚════██║
// ╚███╔███╔╝╚██████╔╝██║  ██║███████╗██████╔╝    ███████╗ ╚████╔╝ ███████╗██║ ╚████║   ██║   ███████║
//  ╚══╝╚══╝  ╚═════╝ ╚═╝  ╚═╝╚══════╝╚═════╝     ╚══════╝  ╚═══╝  ╚══════╝╚═╝  ╚═══╝   ╚═╝   ╚══════╝
const galaxyChests = new Set()

world.afterEvents.entitySpawn.subscribe((data) => {
    if (data.entity.typeId != "kittys:galaxychest") return
    const galaxychest = data.entity
    galaxyChests.add(galaxychest)
    system.runTimeout(() => {
        if (!galaxychest.isValid()) return
        galaxychest.nameTag = `§6Galaxy Crate`
    }, 10)
})

world.beforeEvents.playerInteractWithEntity.subscribe(async (data) => {
    const { player, target } = data
    if (target.typeId != "kittys:galaxychest") return
    const GalaxycrateTimerValue = GalaxySettingsDb.get('KittysGalaxyCrateSettings:GalaxyCrateTimer') || '2';
    const GalaxycrateTimerHours = Number(GalaxycrateTimerValue);
    const cooldownTime = GalaxycrateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    data.cancel = true;
    system.run(() => {
        target.dimension.playSound("random.pop", target.location, { pitch: 0.5 })
    })

    if (
        data.player.hasTag("Admin") ||
        data.player.hasTag("admin") ||
        data.player.hasTag("Staff") ||
        data.player.hasTag("staff") ||
        data.player.hasTag("Owner") ||
        data.player.hasTag("owner")
    ) {
        await GalaxyAdmin_Main_Menu(data.player);
        return;
    }

    function isOnCooldown(player) {
        const now = Date.now();
        if (!Galaxycooldowns.has(player.name)) return false;
        const cooldown = Galaxycooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }

    if (isOnCooldown(data.player)) {
        const remainingTime = cooldownTime - (Date.now() - Galaxycooldowns.get(data.player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the galaxy crate again.✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.player.sendMessage('§6§lOpening Crate Menu!✨');


    const GalaxycrateMenu = new ActionFormData()
        .title('!Galaxy Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.player, GalaxycrateMenu);
        result.selection === 0 ? await GalaxyCrate_Form(data.player) : await GalaxycloseCrateMenu(data.player);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});
// 888    d8P  d8b 888    888                   .d8888b.  888      d8b                        888888b.            d8b 888      888               888
// 888   d8P   Y8P 888    888                  d88P  Y88b 888      Y8P                        888  "88b           Y8P 888      888               888
// 888  d8P        888    888                  Y88b.      888                                 888  .88P               888      888               888
// 888d88K     888 888888 888888 888  888       "Y888b.   88888b.  888 88888888 88888888      8888888K.  888  888 888 888  .d88888 .d8888b       888
// 8888888b    888 888    888    888  888          "Y88b. 888 "88b 888    d88P     d88P       888  "Y88b 888  888 888 888 d88" 888 88K           888
// 888  Y88b   888 888    888    888  888            "888 888  888 888   d88P     d88P        888    888 888  888 888 888 888  888 "Y8888b.      Y8P
// 888   Y88b  888 Y88b.  Y88b.  Y88b 888      Y88b  d88P 888  888 888  d88P     d88P         888   d88P Y88b 888 888 888 Y88b 888      X88       "
// 888    Y88b 888  "Y888  "Y888  "Y88888       "Y8888P"  888  888 888 88888888 88888888      8888888P"   "Y88888 888 888  "Y88888  88888P'      888
//                                    888
//                               Y8b d88P
//                                "Y88P"                                                                                                             